--------------------------------------------------------------------------------
-- Locals

-- Comm stuff
local PRE = "TypI"
local STOP = "stop"

-- Strings
local CMD_SHOW = "Typing indicator visible. Click and drag to position it, and type |cffff4488/typi|r again to hide it."
local CMD_HIDE = "Typing indicator hidden. Type |cffff4488/typi|r again to show it."

local typingMap = {}
local LABEL = "%s is typing ..."

local playerName = UnitName("player")
local lastTarget = nil
local totalElapsed = 0
local updateFrame = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Unit name coloring

local knownPlayers = {}
local noClassColor = "|cffa0a0a0"
local hexColors = {}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = "|cff" .. string.format("%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
end
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = knownPlayers[key] or select(2, UnitClass(key))
		if not class then
			return noClassColor .. key .. "|r"
		else
			self[key] = hexColors[class]  .. key .. "|r"
			return self[key]
		end
	end
})

--------------------------------------------------------------------------------
-- Set up the display

local addon = CreateFrame("Button", "TypingIndicator", UIParent)

addon:SetClampedToScreen(true)
addon:SetHeight(14)
addon:SetWidth(10)
addon:SetFrameStrata("BACKGROUND")
addon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 150, -150)
addon:SetMovable(true)
addon:RegisterForDrag("LeftButton")
addon:SetScript("OnDragStart", addon.StartMoving)
addon:SetScript("OnDragStop", addon.StopMovingOrSizing)
addon:SetScript("OnHide", addon.StopMovingOrSizing)
addon:SetScript("OnEnter", function(self)
	if not next(typingMap) then return end
	GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
	GameTooltip:AddLine("Currently typing:")
	local t = {}
	for player, type in pairs(typingMap) do
		t[#t+1] = coloredName[player]
	end
	GameTooltip:AddLine(table.concat(t, ", "))
	t = nil
	GameTooltip:Show()
end)
addon:SetScript("OnLeave", function(self)
	GameTooltip:Hide()
end)
local indicator = addon:CreateFontString(nil, "OVERLAY", "GameFontNormal")
indicator:SetAllPoints(addon)

local function realSetText(t, type)
	indicator:SetText(t)
	addon:SetWidth(indicator:GetStringWidth())
	if not t or t:len() == 0 then
		addon:Hide()
	else
		addon:Show()
	end
end
local function noop() end
local setText = realSetText

--------------------------------------------------------------------------------
-- Chat command

_G.SlashCmdList["TYPI_SHORTHAND"] = function()
	local t = indicator:GetText()
	if not t or t:len() == 0 then
		setText(LABEL:format(coloredName[playerName]))
		setText = noop
		print(CMD_SHOW)
	else
		setText = realSetText
		setText("")
		print(CMD_HIDE)
	end
end
_G.SLASH_TYPI_SHORTHAND1 = "/typi"

--------------------------------------------------------------------------------
-- Event and comm handling

addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("CHAT_MSG_ADDON")
addon:RegisterEvent("GUILD_ROSTER_UPDATE")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("FRIENDLIST_UPDATE")

function addon:PLAYER_LOGIN()
	if IsInGuild() then GuildRoster() end
	ShowFriends()
end

local function putSomeoneInTheDisplay()
	local player, type = next(typingMap)
	if player then
		local c = ChatTypeInfo[type]
		indicator:SetVertexColor(c.r, c.g, c.b)
		setText(LABEL:format(coloredName[player]))
	else
		setText("")
	end
end

function addon:CHAT_MSG_ADDON(pre, msg, dist, sender)
	if not msg or not sender or pre ~= PRE then return end
	if msg == STOP then
		typingMap[sender] = nil
		putSomeoneInTheDisplay()
	else
		local c = ChatTypeInfo[msg]
		if not c then return end
		typingMap[sender] = msg
		indicator:SetVertexColor(c.r, c.g, c.b)
		setText(LABEL:format(coloredName[sender]))
	end
end
function addon:GUILD_ROSTER_UPDATE()
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, _, _, class = GetGuildRosterInfo(i)
		if name then
			knownPlayers[name] = class
		end
	end
end
function addon:FRIENDLIST_UPDATE()
	for i = 1, GetNumFriends() do
		local name, _, class = GetFriendInfo(i)
		if name then
			knownPlayers[name] = class:upper()
		end
	end
end

--------------------------------------------------------------------------------
-- Track who we're whispering

-- Sends a message to the last tell target to clear the typing indicator
local function clearIndicator()
	if not lastTarget then return end
	if not ChatTypeInfo[lastTarget] then
		SendAddonMessage(PRE, STOP, "WHISPER", lastTarget)
	elseif lastTarget == "GUILD" then
		SendAddonMessage(PRE, STOP, "GUILD")
	else
		SendAddonMessage(PRE, STOP, "RAID")
	end
	lastTarget = nil
	updateFrame:Hide()
end

-- Sends a message to the current tell target to
local function startIndicator(target)
	lastTarget = target
	if not ChatTypeInfo[target] then
		SendAddonMessage(PRE, "WHISPER", "WHISPER", target)
	elseif target == "GUILD" then
		SendAddonMessage(PRE, target, "GUILD")
	else
		SendAddonMessage(PRE, target, "RAID")
	end
	updateFrame:Show()
end

-- OnUpdate function to handle a typing timeout, if nothing is typed for 5
-- seconds, we send a typing indicator clear to the last tell target.
updateFrame:SetScript("OnShow", function()
	totalElapsed = 0
end)
updateFrame:SetScript("OnUpdate", function(self, elapsed)
	if not lastTarget then return end
	totalElapsed = totalElapsed + elapsed
	if totalElapsed > 5 then
		clearIndicator()
	end
end)
updateFrame:Hide()

hooksecurefunc("ChatEdit_OnTextChanged", function(self, userInput)
	local text = self:GetText()
	if not text or text:len() < 1 or text:sub(1, 1) == "/" then return end

	local type = self:GetAttribute("chatType")
	if not type then return end

	local target = nil
	if type == "WHISPER" then
		target = self:GetAttribute("tellTarget")
	elseif type == "PARTY" or type == "RAID" or type == "GUILD" then
		target = type
	else
		return
	end
	if (target == "RAID" or target == "PARTY") and UnitInBattleground("player") then return end

	if not target or (lastTarget and target ~= lastTarget) then
		clearIndicator()
	end
	if target and not lastTarget then
		startIndicator(target)
	end
	-- Reset the timer so we have another 5 seconds to type again before sending the clearIndicator
	totalElapsed = 0
end)

hooksecurefunc("ChatEdit_OnEscapePressed", function(self)
	clearIndicator()
end)

